%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script allows to evaluate the software BANJO
% by testing it on a set of randomly generated in silico networks with
% varying levels of noise in the corresponding measurement data.
% To this aim, the following  functions have been used:
% banjo_results.m and ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc

net_nodes=20; % test over the networks of twenty nodes

% networks number
n_networks=20;

% number of tests: for each network, we consider a set of data of 25 time 
% points, assuming two levels of noise (10%, 30%) and two level of prior
% knowledge (PK) (10% and 20%)
n_tests=4;

n_best_net=5; % number of the highest scoring networks for BANJO


A_id_cell_Banjo{n_networks,n_tests}=[]; % cell array containing the 
                                        % top-scoring network, for each 
                                        % test  

                                      

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    eval([' load results_net_in_silico_20n\res_banjo_',num2str(net_nodes),'n'])   
end

% define a vector to store the sparsity coefficient for each network
spars_networks=zeros(1,n_networks);

% define matrices to store the results 
PPV_Banjo=zeros(n_networks,n_tests);
Se_Banjo=zeros(n_networks,n_tests);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%  COMPUTE THE PERFORMANCE OF BANJO   %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_net=1:n_networks 
    % load the data for the current network
    eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
        num2str(num_net)]);
    spars_networks(num_net)=spar_coef;
    
    for num_test=1:n_tests
        % num_test=1, corresponding to the results obtained by using a data
        %             set of 25 time points, 10% level of noise, 10% level of
        %             prior knowledge             
        % num_test=2, corresponding to the results obtained by using a data
        %             set of 25 time points, 30% level of noise, 10% level of
        %             prior knowledge 
        % num_test=3, corresponding to the results obtained by using a data
        %             set of 25 time points, 10% level of noise, 20% level of
        %             prior knowledge 
        % num_test=4, corresponding to the results obtained by using a data
        %             set of 25 time points, 30% level of noise, 20% level of
        %             prior knowledge        
        if num_test>=1 && num_test<=2
            str_pk='_pk_10';
        elseif num_test>2 && num_test<=4
            str_pk='_pk_20';
        end
           
        if do_sim
            
            % run BANJO
            % Compute the top-scoring network corresponding to the
            % connectivity matrix
            eval(['dos(''java -jar banjo.jar settingsFile=data\'...
                'release2.0\dynamic\Banjo_dyn_sett_file_net_in_silico_',...
                num2str(net_nodes),'n\dyn_sett_sys_',...
                num2str(net_nodes),'n_data_',str_level_noise,str_pk,...
                '_net_',num2str(num_net),'.txt'')']);
            
            
            % define a string containing the path of the report file
            % returned by Banjo
            filename=['data\release2.0\dynamic\output'...
                '\Banjo_results_net_in_silico_',num2str(net_nodes),'n'...
                '\results_',num2str(net_nodes),'n_data_',...
                str_level_noise,str_pk,'_net_',num2str(num_net),'.txt'];

            [A_id_best_Banjo]=banjo_results(filename,n_nodes,...
                n_best_net,false);

            % store the cell array that contains the best identified
            % connectivity matrices for each network and test
            A_id_cell_Banjo{num_net,num_test}=A_id_best_Banjo;
       
        else 
            A_id_best_Banjo=A_id_cell_Banjo{num_net,num_test};
        end
        
            
        % evaluation of the performance by calculating the PPV and
        % sensitivity indeces
        Perf_idx= ppv_sens_spec(original_network,A_id_best_Banjo{1});

        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % save in two matrix (PPV_Banjo, Se_Banjo) the PPV and Sensitivity
        % (Se) performance indeces: the row index corresponds to the
        % network number, the column index corresponds to the kind of
        % test
        PPV_Banjo(num_net,num_test)=Perf_idx(2,1);
        Se_Banjo(num_net,num_test)=Perf_idx(2,2);
    end
end


x_pr_v=linspace(0,1,100);

spar_coef_mean=mean(spars_networks);
PK_l=0.1;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_10=PK_l*(1-spar_coef_mean)+(1-PK_l)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir_10=x_pr_v*spar_coef_mean;
FP_dir_10=(1-x_pr_v)*spar_coef_mean;
FN_dir_10=(1-PK_l)*x_pr_v*(1-spar_coef_mean);
sens_dir_10=TP_dir_10./(TP_dir_10+FN_dir_10);
% spec_dir_10=TN_dir./(TN_dir_10+FP_dir_10);
PPV_dir_10=TP_dir_10./(TP_dir_10+FP_dir_10);



% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV, Spec, Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef_mean)+(1-PK_l)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef_mean;
FP_dir_20=(1-x_pr_v)*spar_coef_mean;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef_mean);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);



figure
for count=1:4
    s=subplot(2,2,count);
    hold on;
    grid on
    set(s,'FontSize',12)
    axis([0.1 0.4 0.1 1 ]);
    if count==1
        num_test=1; % 10% level of noise, 10% of PK
        title('PK=10%','FontSize',12);
        ylabel(['10% noise',sprintf('\n'),'PPV'],'FontSize',12);
        plot(sens_dir_10,PPV_dir_10,'r-','LineWidth',2);
        axis([0.1 0.4 0.1 1 ]);

    elseif count==2
        num_test=3; % 10% level of noise, 20% of PK
        title('PK=20%','FontSize',12);
        plot(sens_dir_20,PPV_dir_20,'r-','LineWidth',2);
        axis([0.1 0.4 0.1 1 ]);


    elseif count==3
        num_test=2; % 30% level of noise, 10% of PK
        ylabel(['30% noise',sprintf('\n'),'PPV'],'FontSize',12);
        plot(sens_dir_10,PPV_dir_10,'r-','LineWidth',2);
        xlabel('Sn','FontSize',12)
        axis([0.1 0.4 0.1 1 ]);

    elseif count==4
        num_test=4; % 30% level of noise, 20% of PK
        plot(sens_dir_20,PPV_dir_20,'r-','LineWidth',2);
        xlabel('Sn','FontSize',12)
        axis([0.1 0.4 0.1 1 ]);
    end

    col_b='k';
    plot(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),'MarkerEdgeColor',col_b,...
        'MarkerSize',8,'Marker','d','LineWidth',2,'LineStyle','none');
    herr3=herrorbar(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),std(Se_Banjo(:,num_test)),col_b);
    set(herr3,'LineWidth',1);
    errorbar(mean(Se_Banjo(:,num_test)),mean(PPV_Banjo(:,num_test)),std(PPV_Banjo(:,num_test)),col_b,'LineWidth',1);
end

leg1=legend('RANDOM','BANJO','Orientation','horizontal');
% set(leg1,'OuterPosition',[0.1 0.02 0.4 0.04]);
% set(leg1,'Position',[0.07591 0.02692 0.8808 0.06018]);
% g=[1 1 816 651];
% 
% set(gcf,'OuterPosition',g)
