%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to generate the input files (the data and the 
% constraints to be imposed) that can be used by the software BANJO for
% reconstructing the in-silico networks of twenty nodes with
% desired topology, generated by using the scalefree network generation 
% model proposed by Albert and Barabasi.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks= 20;

% number of tests: for each network, we consider a set of data of 25 time 
% points, assuming two levels of noise (10%, 30%) and two levels of 
% prior knowledge (PK) (10% and 20%)
n_tests=4;

for num_net=1:n_networks
    
    % load the data for the current network
    eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
        num2str(num_net)]);
    
    % define the file names 
    filename_data_noise10=['C:\Users\Francesco\Documents\MATLAB'...
        '\PACTLS code\PACTLS code\Banjo_data_net_in_silico_',...
        num2str(net_nodes),'n\sys_',num2str(net_nodes),...
        'n_data_noise10_net_',num2str(count),'.txt'];    
    filename_data_noise30=['C:\Users\Francesco\Documents\MATLAB'...
        '\PACTLS code\PACTLS code\Banjo_data_net_in_silico_',...
        num2str(net_nodes),'n\sys_',num2str(net_nodes),...
        'n_data_noise30_net_',num2str(count),'.txt'];
    
    % writes data matrices into files (.txt)
    dlmwrite(filename_data_noise10, Meas_mtx_25tp_noise10,...
        'delimiter', '\t','precision', 6)
    dlmwrite(filename_data_noise30, Meas_mtx_25tp_noise30,...
        'delimiter', '\t','precision', 6)


    
    for num_test=1:n_tests 
        
        if num_test>=1 && num_test<=2  
            Constr_mtx_in=Constr_mtx_10;
            str_pk='_pk_10';
        elseif num_test>2 && num_test<=4
            Constr_mtx_in=Constr_mtx_20;
            str_pk='_pk_20';
        end
    
    
        % generate the file.str containing the specifies edges to be imposed
        filename_prior=['C:\Users\Francesco\Documents\MATLAB\PACTLS code\'...
            'PACTLS code\Banjo_data_net_in_silico_',...
            num2str(net_nodes),'n\sys_',...
            num2str(net_nodes),'n',str_pk,'_',num2str(count),'.str'];

        % define the prior known network 
        prior_orig_net=eye(n_nodes);
    
        for idx1=1:size(Constr_mtx_in,1)
            prior_orig_net(Constr_mtx_in(idx1,1),...
                Constr_mtx_in(idx1,2))=Constr_mtx_in(idx1,3);
        end

        % create the file.str
        fid = fopen(filename_prior, 'a+');
        fprintf(fid,'%d\n',n_nodes);
        fclose(fid)
        for idx2=1:n_nodes        
            fid = fopen(filename_prior, 'a+');
            % find the in-coming edges for each nodes
            link_from_nodes=find(prior_orig_net(idx2,:)); 
            vct_data=[idx2-1, 0, 0, 1, length(link_from_nodes),...
                link_from_nodes-1];
            str_stc='%d %d: %d %d: %d';
            str_dyn=[];
            for idx3=1:length(link_from_nodes)
                str_dyn_old=str_dyn;
                if idx3==length(link_from_nodes)
                    str_dyn=[str_dyn_old,' %d\n'];
                else
                    str_dyn=[str_dyn_old,' %d'];
                end
                clear str_dyn_old
            end
            format_str=[str_stc, str_dyn];
            clear str_stc str_dyn
            fprintf(fid,format_str,vct_data);
            fclose(fid)
        end
    end
end
