function [err_pr err_dy]=stima_err_pr_dy(Rec_mtx,Meas_mtx,num_exp)

% This function computes the state evolution of the identified model and 
% compare with experimental measurements
%
% INPUT VARIABLES
% 
% Rec_mtx: reconstructed matrix, containing the identified connectivity 
%          matrix and exogenous perturbation vector B 
%           
% Meas_mtx:  Measurements matrix, each column contains the measurements for
%            a node
%
% num_exp:    Number of experiments
%
% OUTPUT VARIABLES 
%
% err_pr: prediction error
%
% err_dy: "dynamic error" (obtained by evolving the identified model) 


[n_time_pts,n_nodes]=size(Meas_mtx);
% Matrix of the measurements at time point k from 1 to n_time_pts-1
Omega_mtx=Meas_mtx(1:n_time_pts-1,:)';
Omega_mtx=[Omega_mtx;kron(eye(num_exp),ones(1,n_time_pts-1))];
% Matrix of the measurements at time point k+1 from 2 to n_time_pts
Theta_mtx=Meas_mtx(2:n_time_pts,:)';


% Compute the prediction error 
% Compute the state evolution of the identified model and compare
% with experimental measurements
Xsim=Rec_mtx*Omega_mtx;

% Compute the absolute error matrix
Err_mtx=abs(Theta_mtx-Xsim);

% Normalization of the error matrix
% Find the maximum absolute value in the matrix
max_Theta_mtx=max(max(abs(Theta_mtx)));
% Divide each element by the abs value of the corresponding experimental
% measurement
for idx1=1:size(Theta_mtx,1)
    % The normalized error is computed only on the values that are at
    % least 10% of the maximum of the signal and only on those signals
    % whose maximum is at least 10% of the maximum over all the signals
    if max(abs(Theta_mtx(idx1,:)))>max_Theta_mtx*0.1;
        row_thr=0.1*max(abs(Theta_mtx(idx1,:)));
    else
        % Do not consider this row in the computation of the estimation
        % error
        row_thr=Inf;
    end
    for idx2=1:size(Theta_mtx,2)
        if abs(Theta_mtx(idx1,idx2))>row_thr
            Err_mtx(idx1,idx2)=Err_mtx(idx1,idx2)/abs(Theta_mtx(idx1,idx2));
        else
            Err_mtx(idx1,idx2)=0;
        end
    end
end
% Compute the cumulative error for each signal, pick the largest one and
% divide by the number of time points
err_pr=norm(Err_mtx',1)/n_time_pts;
 

% compute the "dynamic" error

X0=Omega_mtx(1:n_nodes,1);
Xsim_dy(:,1)=X0;
for idx=2:n_time_pts
    Theta_sim= Rec_mtx(1:n_nodes,1:n_nodes)*X0+Rec_mtx(1:n_nodes,n_nodes+num_exp);
    Xsim_dy(:,idx)=Theta_sim;
    X0=Theta_sim;
end
% Compute the absolute error matrix
Err_mtx_dy=abs(Meas_mtx-Xsim_dy');

% Normalization of the error matrix
% Find the maximum absolute value in the matrix
max_Meas_mtx=max(max(abs(Meas_mtx)));
% Divide each element by the abs value of the corresponding experimental
% measurement
for idx2=1:size(Meas_mtx,2)
    % The normalized error is computed only on the values that are at
    % least 10% of the maximum of the signal and only on those signals
    % whose maximum is at least 10% of the maximum over all the signals
    if max(abs(Meas_mtx(:,idx2)))>max_Meas_mtx*0.1;
        col_thr=0.1*max(abs(Meas_mtx(:,idx2)));
    else
        % Do not consider this column in the computation of the estimation
        % error
        col_thr=Inf;
    end
    for idx1=1:size(Meas_mtx,1)
        if abs(Meas_mtx(idx1,idx2))>col_thr
            Err_mtx_dy(idx1,idx2)=Err_mtx_dy(idx1,idx2)/abs(Meas_mtx(idx1,idx2));
        else
            Err_mtx_dy(idx1,idx2)=0;
        end
    end
end
% define the "dynamic" error
err_dy=norm(Err_mtx_dy,1)/n_time_pts;

